# Generated by Django 3.1.3 on 2020-11-23 14:42

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('auth', '0012_alter_user_first_name_max_length'),
        ('sso_auth', '0003_oauthuser_groups'),
    ]

    operations = [
        migrations.AddField(
            model_name='oauthuser',
            name='user_permissions',
            field=models.ManyToManyField(blank=True, related_name='user_set', related_query_name='user', to='auth.Permission', verbose_name='user permissions'),
        ),
    ]



import json
import requests
from django.conf import settings


class GetTokenByRefresh(object):
	"""
	request to sso server for get access token by refresh
	"""
	@staticmethod
	def get_token(refresh_token: str) -> dict:
		url = f'{settings.SSO_URL}token'
		
		data = {
			'grant_type': 'refresh_token',
			'client_id': settings.CLIENT_ID,
			'client_secret': settings.CLIENT_SECRET,
			'redirect_uri': settings.REDIRECT_URI,
			'refresh_token': refresh_token
		}

		response = requests.post(url, data=data)
		
		result = {}
		
		if response.status_code == 200:
			result = json.loads(response.text)
			
		return result



# Generated by Django 3.1.3 on 2020-11-23 11:11

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('sso_auth', '0001_initial'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='oauthuser',
            name='groups',
        ),
        migrations.RemoveField(
            model_name='oauthuser',
            name='user_permissions',
        ),
    ]



from typing import Optional
from django.contrib.auth.backends import BaseBackend
from django.contrib.auth.models import Group
from .models import OauthUser
from sso_auth.services.get_user_info import GetUserInfo


class TokenAuth(BaseBackend):
	@staticmethod
	def authenticate(request, **kwargs):
		if request is not None:
			if request.COOKIES.get('access_token', False):
				return TokenAuth.get_user(request.COOKIES.get('access_token'))
				
		return None
	
	@staticmethod
	def get_user(access_token: str) -> Optional[OauthUser]:
		info = GetUserInfo.get_info(access_token)
		
		if info.get('user_id'):
			user = OauthUser()
			
			user.id = info.get('user_id')
			user.identifier = info.get('username')
			
			if 'is_staff' in info.get('groups'):
				user.is_staff = True
			else:
				user.is_staff = False
			
			if 'is_superuser' in info.get('groups'):
				user.is_superuser = True
			else:
				user.is_superuser = False
			
			user.is_active = info.get('is_active')
			user.is_anonymous = False
			user.email = info.get('email')
			
			groups = Group.objects.filter(name__in=info.get('groups'))
			
			user.groups.clear()
			user.user_permissions.clear()
			
			for group in groups:
				permissions = group.permissions.all()

				for permission in permissions:
					user.user_permissions.add(permission.id)
				user.groups.add(group.id)
				
			user.save()
			
			return user
		
		return None



import json
import requests
from django.conf import settings


class GetUserInfo(object):
	"""
	request to sso server to get user's info
	"""
	@staticmethod
	def get_info(access_token: str) -> dict:
		url = f'{settings.SSO_URL}info'
		
		headers = {
			'Authorization': 'Bearer {}'.format(access_token)
		}
		
		response = requests.get(url, headers=headers)
		
		if response.status_code == 200:
			info = json.loads(response.text)
			return info
		else:
			return {}



import setuptools

with open("README.md", "r") as fh:
	long_description = fh.read()

setuptools.setup(
	name="ksu_sso_auth",  # Replace with your own username
	version="0.1.0",
	author="Dmitry Shoytov",
	author_email="shoytov@gmail.com",
	description="Module for use ksu sso service ",
	long_description=long_description,
	long_description_content_type="text/markdown",
	url="https://gitlab.com/shoytov/sso_auth",
	packages=setuptools.find_packages(),
	package_data={'sso_auth.templates': '*', 'sso_auth.static': '*'},
	install_requires=[
		'requests',
	],
	classifiers=[
		"Programming Language :: Python :: 3",
		"License :: OSI Approved :: MIT License",
		"Operating System :: OS Independent",
	],
	python_requires='>=3.6',
)



import json
from django.views.generic import View
from django.shortcuts import redirect, render
from django.template.loader import render_to_string
from django.conf import settings
from django.http import HttpResponse, HttpResponseRedirect
import requests
from sso_auth.services.get_user_info import GetUserInfo
from sso_auth.services.get_token_by_refresh import GetTokenByRefresh


url = f'{settings.SSO_URL}authorize?response_type=code&client_id={settings.CLIENT_ID}&redirect_uri={settings.REDIRECT_URI}'


# Create your views here.
class AdminLogin(View):
	"""
	redefine auth form for admin panel
	"""
	def get(self, request):
		if not request.COOKIES.get('access_token', False) or not request.COOKIES.get('refresh_token', False):
			return redirect(url)
		else:
			response = HttpResponseRedirect('/admin')
			
			info = GetUserInfo.get_info(request.COOKIES.get('access_token'))

			if not info.get('user_id'):
				# try get new access token by refresh token
				tokens = GetTokenByRefresh.get_token(request.COOKIES.get('refresh_token'))
				
				if tokens.get('access_token'):
					print(tokens.get('access_token'))
					info = GetUserInfo.get_info(tokens.get('access_token'))
					
				if not info.get('user_id') or not info.get('is_active'):
					response.delete_cookie('access_token')
					response.delete_cookie('refresh_token')
				else:
					response.set_cookie('access_token', tokens.get('access_token'))
					response.set_cookie('refresh_token', tokens.get('refresh_token'))
				
			return response
	
			
class UserLogout(View):
	"""
	user's logout
	"""
	def get(self, request):
		response = HttpResponseRedirect('https://sso.kursksu.ru/admin/logout/')
		response.delete_cookie('access_token')
		response.delete_cookie('refresh_token')
		
		return response
	
		
class ProcessRedirectUri(View):
	"""
	process grant code to receive access token
	"""
	def get(self, request):
		if not request.GET.get('code'):
			return redirect(url)
		
		data = {
			'grant_type': 'authorization_code',
			'client_id': settings.CLIENT_ID,
			'client_secret': settings.CLIENT_SECRET,
			'redirect_uri': settings.REDIRECT_URI,
			'code': request.GET.get('code')
		}
		
		response = requests.post(f'{settings.SSO_URL}token', data=data)
		
		if response.status_code == 200:
			token = json.loads(response.text)
			
			content = render_to_string('success_auth.html')
			
			response = HttpResponse(content)
			response.set_cookie('access_token', token.get('access_token'))
			response.set_cookie('refresh_token', token.get('refresh_token'))
			
			return response



from django.utils.deprecation import MiddlewareMixin
from .backends import TokenAuth


class OauthMiddleware(MiddlewareMixin):
	def process_request(self, request) -> None:
		if request.COOKIES.get('access_token', False):
			user = TokenAuth.authenticate(request=request)
			if user:
				request.user = request._cached_user = user



from django.db import models
from django.contrib.auth.models import AbstractBaseUser, PermissionsMixin, Group, Permission
from django.utils.translation import gettext_lazy as _


# Create your models here.
class OauthUser(AbstractBaseUser, PermissionsMixin):
	identifier = models.CharField(max_length=40, unique=True)
	
	email = None
	is_staff = None
	is_active = None
	is_superuser = None
	is_anonymous = True
	
	groups = models.ManyToManyField(
			Group,
			verbose_name=_('groups'),
			blank=True,
			related_name="oauth_user_set",
			related_query_name="oauth_user"
	)
	
	user_permissions = models.ManyToManyField(
			Permission,
			verbose_name=_('user permissions'),
			blank=True,
			related_name="user_set",
			related_query_name="user"
	)
	
	USERNAME_FIELD = 'identifier'

	def __str__(self):
		return self.identifier
	
	def has_module_perms(self, app_label):
		permissions = self.user_permissions.all()
		return app_label in [f'{permission.content_type.app_label}' for permission in permissions]
	
	def has_perm(self, perm, obj=None):
		permissions = self.user_permissions.all()
		return perm in [f'{permission.content_type.app_label}.{permission.codename}' for permission in permissions]
